#!/usr/bin/env python

import numpy as np
from matplotlib import animation
from matplotlib import pyplot as plt
from matplotlib.backends.qt_compat import QtWidgets
from matplotlib.widgets import Slider

π = np.pi


def lissajoux(tt, φx=0., φy=0., ωx=1., ωy=1., dω=0., dφ=0.):
    x = np.cos(2 * π * (ωx * tt + φx))
    y = np.cos(2 * π * ((ωy + dω) * tt + φy + dφ / 2))
    return x, y


def update(update=False):
    ωx = ωx_slider.val
    ωy = ωy_slider.val
    dω = 1e-3 * dω_slider.val
    if update:
        global φx, φy, i
        φx += ωx
        φy += ωy + dω
        i += vm_slider.val
        i %= nt
    x, y = lissajoux(tt, φx, φy, ωx, ωy, 1e-3 * dω, dφ_slider.val)
    linel.set_data(x, y)
    linex.set_ydata(x)
    liney.set_ydata(y)
    markl.set_data(x[i], y[i])
    markx.set_data(tt[i], x[i])
    marky.set_data(tt[i], y[i])


plt.style.use('dark_background')
fig = plt.figure(figsize=(16, 9))
manager = fig.canvas.manager
manager.window.findChild(QtWidgets.QToolBar).setVisible(False)
manager.set_window_title("Lissajoux")
manager.full_screen_toggle()

gs = fig.add_gridspec(nrows=9, ncols=16, hspace=0.05, top=0.98, bottom=0.02, left=0.02, right=0.98)
axl = fig.add_subplot(gs[:, :9])
axt = fig.add_subplot(gs[:4, 9:])

axl.set_aspect('equal')
axl.xaxis.set_major_locator(plt.MultipleLocator(1.00))
axl.xaxis.set_minor_locator(plt.MultipleLocator(0.25))
axl.yaxis.set_major_locator(plt.MultipleLocator(1.00))
axl.yaxis.set_minor_locator(plt.MultipleLocator(0.25))
axl.grid(True, which='major', color='0.7', lw=0.7)
axl.grid(True, which='minor', color='0.3', lw=0.7)
axl.tick_params(axis='both', which='both', direction="in", left=True, right=True, top=True, bottom=True)
axl.xaxis.set_major_formatter(plt.NullFormatter())
axl.yaxis.set_major_formatter(plt.NullFormatter())

axt.xaxis.set_major_locator(plt.MultipleLocator(0.25))
axt.xaxis.set_minor_locator(plt.MultipleLocator(0.05))
axt.yaxis.set_major_locator(plt.MultipleLocator(1.00))
axt.yaxis.set_minor_locator(plt.MultipleLocator(0.25))
axt.grid(True, axis='x', which='major', color='0.6', lw=0.7)
axt.grid(True, axis='x', which='minor', color='0.2', lw=0.7)
axt.grid(True, axis='y', which='major', color='0.7', lw=0.7)
axt.grid(True, axis='y', which='minor', color='0.2', lw=0.7)
axt.tick_params(axis='y', which='both', direction="in", left=True, right=True)
axt.yaxis.set_major_formatter(plt.NullFormatter())

φx = φy = 0
tt = np.linspace(0., 1., 1001) - 0.5
x, y = lissajoux(tt)

i = 0
nt = len(tt)

linel, = axl.plot(x, y, lw=2.0, color='#feffb3')
linex, = axt.plot(tt, x, lw=1.5, color='#81b1d2')
liney, = axt.plot(tt, y, lw=1.5, color='#fa8174')
markl, = axl.plot(x[i], y[i], 'o', ms=10, color='#bc82bd')
markx, = axt.plot(tt[i], x[i], 'o', ms=5, color='#81b1d2')
marky, = axt.plot(tt[i], y[i], 'o', ms=5, color='#fa8174')

axl.set_xlim(-1.05, 1.05)
axl.set_ylim(-1.05, 1.05)

axt.set_xlim(tt[0], tt[-1])
axt.set_ylim(-1.20, 1.20)

axt.set_xlabel('$t$ [s]')

fig.tight_layout()

# fig.subplots_adjust(right=0.7)

ωxax = fig.add_axes([0.64, 0.45, 0.30, 0.03])
ωyax = fig.add_axes([0.64, 0.35, 0.30, 0.03])
dφax = fig.add_axes([0.64, 0.25, 0.30, 0.03])
dωax = fig.add_axes([0.64, 0.15, 0.30, 0.03])
vmax = fig.add_axes([0.64, 0.05, 0.30, 0.03])

sst = lambda color='0.3': {
    'track_color': color,
    'color': color,
    'initcolor': 'black',
}

ωx_slider = Slider( ax=ωxax, label='$\\omega_x$ [Hz]', valmin=1, valmax=12, valinit=1, valstep=1, **sst('#81b1d2'))
ωy_slider = Slider(ax=ωyax, label='$\\omega_y$ [Hz]', valmin=1, valmax=12, valinit=1, valstep=1, **sst('#fa8174'))
dφ_slider = Slider(ax=dφax, label='$\\Delta\\varphi$ [$\\pi$]', valmin=-1.0, valmax=1.0, valstep=0.125, valinit=0., **sst())
dω_slider = Slider(ax=dωax, label='$\\Delta\omega$ [kHz]', valmin=-1.00, valmax=1.00, valstep=0.05, valinit=0., **sst())
vm_slider = Slider(ax=vmax, label='$v_m$', valmin=1, valmax=10, valstep=1, valinit=2, **sst())


def on_slider(val):
    linex.set_xdata(tt)
    liney.set_xdata(tt)
    update()
    fig.canvas.draw_idle()


ωx_slider.on_changed(on_slider)
ωy_slider.on_changed(on_slider)
dω_slider.on_changed(on_slider)
dφ_slider.on_changed(on_slider)


def animate(i):
    update(True)
    return linel, linex, liney, markl, markx, marky


anim = animation.FuncAnimation(fig, animate, interval=10, blit=True)

plt.show()
